--== Options ==--
-- true to enable; false to disable
local background = true --little background behind the unit frames
local healthtext_opt = true --text for health or not
local powertext_opt = true -- text for power or not
local leader = true -- leader icon or not
local RI = true -- raid icons or not
local name_opt = true -- name text or not
local playerCB = true -- player castbar or not
local targetCB = true -- target castbar or not
local target_debuff = true -- target debuffs or not
local target_buff = true -- target buffs or not
local reputation = false
local experience = false

local function Hex(r, g, b)
	if type(r) == "table" then
		if r.r then r, g, b = r.r, r.g, r.b else r, g, b = unpack(r) end
	end
	return string.format("|cff%02x%02x%02x", r*255, g*255, b*255)
end

--== Textures Options ==--
local font, fontsize = "Fonts\\FRIZQT__.ttf", 12		-- The font and fontsize


local statusbar = [[Interface\Addons\oUF_Deith\media\pHishTex15]]
local glowTex = [[Interface\Addons\oUF_Deith\media\glowTex]]
local backdrop = {
	bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
	insets = {top = -1, left = -1, bottom = -1, right = -1},
}
local playerClass = select(2, UnitClass("player"))
local iconTex = [[Interface\AddOns\oUF_Deith\media\gloss5]]
--== End of textures options ==--


--== Custom colors ==--
local colors = setmetatable({
	power = setmetatable({
		['MANA'] = {0, 252/255, 1}
	}, {__index = oUF.colors.power}),
}, {__index = oUF.colors})


local runeloadcolors = {
	[1] = {.69,.31,.31},
	[2] = {.69,.31,.31},
	[3] = {.33,.59,.33},
	[4] = {.33,.59,.33},
	[5] = {.31,.45,.63},
	[6] = {.31,.45,.63},
}

--== The function to create font strings, thanks to Caellian ==--
local function SetFontString(parent, fontName, fontHeight, fontStyle)
	local fs = parent:CreateFontString(nil, 'OVERLAY')
	fs:SetFont(fontName, fontHeight, fontStyle)
	fs:SetJustifyH('LEFT')
	fs:SetShadowColor(0,0,0)
	fs:SetShadowOffset(1.25, -1.25)
	return fs
end


local menu = function(self)
	local unit = self.unit:sub(1, -2)
	local cunit = self.unit:gsub("(.)", string.upper, 1)

	if(unit == "party" or unit == "partypet") then
		ToggleDropDownMenu(1, nil, _G["PartyMemberFrame"..self.id.."DropDown"], "cursor", 0, 0)
	elseif(_G[cunit.."FrameDropDown"]) then
		ToggleDropDownMenu(1, nil, _G[cunit.."FrameDropDown"], "cursor", 0, 0)
	end
end 

--== Skin the buttons (buffs and debuffs) ==--
local function updateDebuff(self, icons, unit, icon, index)
	local name, _, _, _, dtype = UnitAura(unit, index, icon.filter)

	if(icon.debuff) then
		if(not UnitIsFriend('player', unit) and icon.owner ~= 'player' and icon.owner ~= 'vehicle') then
			icon:SetBackdropColor(0, 0, 0)
			icon.icon:SetDesaturated(true)
		else
			local color = DebuffTypeColor[dtype] or DebuffTypeColor.none
			icon:SetBackdropColor(color.r * 0.6, color.g * 0.6, color.b * 0.6)
			icon.icon:SetDesaturated(false)
		end
	end
end

local function createAura(self, button, icons)
	icons.showDebuffType = true

	button.cd:SetReverse()
	button.cd.noOCC = true
	button.cd.noCooldownCount = false
	icons.disableCooldown = false
	
	button.count:SetPoint("BOTTOMRIGHT", 1, 1.5)
	button.count:SetJustifyH("RIGHT")
	button.count:SetFont(font, 8)
	button.count:SetTextColor(0.84, 0.75, 0.65)
	
	button:SetBackdrop({
	  edgeFile = glowTex, edgeSize = 8,
	  insets = {left = 3, right = 3, top = 3, bottom = 3}
	})
	button:SetBackdropColor(0, 0, 0)
	
	button.icon:SetTexCoord(0.1, 0.9, 0.1, 0.9)
	button.icon:SetDrawLayer('ARTWORK')
	
	button.overlay:SetTexture(iconTex)
	button.overlay:SetPoint("TOPLEFT", button, "TOPLEFT", -2, 2)
	button.overlay:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 2, -2)
	button.overlay:SetTexCoord(0, 1, 0.02, 1)
	button.overlay.Hide = function(self) end
end


--== The Druid mana ==--
local function UpdateDruidPower(self, event, unit)
	if(unit and unit ~= self.unit) then return end

	local text = self.DruidPower.Text
	local num, str = UnitPowerType('player')
	local min = UnitPower('player', (num ~= 0) and 0 or 3)
	local max = UnitPowerMax('player', (num ~= 0) and 0 or 3)

	--bar:SetMinMaxValues(0, max)

	if(min ~= max) then
		--bar:SetValue(min)
		--bar:SetAlpha(1)

		if(num ~= 0) then
			--bar:SetStatusBarColor(unpack(colors.power['MANA']))
			text:SetFormattedText('%d - %d%%', min, math.floor(min / max * 100))
		else
			--bar:SetStatusBarColor(unpack(colors.power['ENERGY']))
			text:SetText()
		end
	else
		--bar:SetAlpha(0)
		text:SetText()
	end
end

--== The layout really begins here ==--
local function Style(self, unit)
	self.colors = colors
	self.menu = menu
	self:RegisterForClicks('AnyUp')
	self:SetAttribute('*type2', 'menu')
	self:SetScript('OnEnter', UnitFrame_OnEnter)
	self:SetScript('OnLeave', UnitFrame_OnLeave)
	
	--== Unit Backgrounds ==--
	self:SetBackdrop(backdrop)
	self:SetBackdropColor(0,0,0,1)
	
	self.FrameBackdrop = CreateFrame("Frame", nil, self)
	self.FrameBackdrop:SetPoint("TOPLEFT", self, "TOPLEFT", -4.5, 4)
	self.FrameBackdrop:SetFrameStrata("BACKGROUND")
	self.FrameBackdrop:SetBackdrop {
	  edgeFile = glowTex, edgeSize = 8,
	  insets = {left = 3, right = 3, top = 3, bottom = 3}
	}
	self.FrameBackdrop:SetBackdropColor(0, 0, 0, 0)
	self.FrameBackdrop:SetBackdropBorderColor(0, 0, 0)

	if unit == "player" and class == "DEATHKNIGHT" or IsAddOnLoaded("oUF_TotemBar") and unit == "player" and class == "SHAMAN" then
		self.FrameBackdrop:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", 4.5, -12)
	else
		self.FrameBackdrop:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", 4.5, -4.5)
	end


--== Plugins part ==--
	--== Movable frames ==--
	self.MoveableFrames = true
	--== BarFader options ==--
	self.BarFade = true
	self.BarFadeMinAlpha = 0.05

	--== Debuff Highlight ==--
	self.DebuffHighlightBackdrop = true


--[[
TODO: Add other plugins support and change those ones
--]]

	--== RuneBar ==--
if unit == "player" and playerClass == "DEATHKNIGHT" then
	self.Runes = CreateFrame("Frame",nil, self)
	self.Runes:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 0, -1)
	self.Runes:SetHeight(7)
	self.Runes:SetWidth(230)
	self.Runes.height = 7
	self.Runes.width = 230/6 - .85
	self.Runes.spacing = 1
	self.Runes.anchor = "TOPLEFT"
	self.Runes.growth = "RIGHT"
	
	for i = 1,6 do 
		self.Runes[i] = CreateFrame("StatusBar",nil, self.Runes)
		self.Runes[i]:SetStatusBarTexture(statusbar)
		self.Runes[i]:SetStatusBarColor(unpack(runeloadcolors[i]))
		
		self.Runes[i].bg = self.Runes[i]:CreateTexture(nil, "BORDER")
		self.Runes[i].bg:SetAllPoints(self.Runes[i])
		self.Runes[i].bg:SetTexture(statusbar)
		self.Runes[i].bg:SetVertexColor(0.15, 0.15, 0.15)
	end
end	


	--== TotemBar ==--

if IsAddOnLoaded("oUF_TotemBar") and unit == "player"  and playerClass == "SHAMAN" then
	self.TotemBar = {}
	self.TotemBar.Destroy = true
	for i = 1, 4 do
		self.TotemBar[i] = CreateFrame("StatusBar", "$parent_TotemBar"..i, self)
		self.TotemBar[i]:SetHeight(7)
		self.TotemBar[i]:SetWidth(230/4 - 0.85)
		if (i == 1) then
			self.TotemBar[i]:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 0, -1)
		else
			self.TotemBar[i]:SetPoint("TOPLEFT", self.TotemBar[i-1], "TOPRIGHT", 1, 0)
		end
		self.TotemBar[i]:SetStatusBarTexture(statusbar)
		self.TotemBar[i]:SetBackdrop(backdrop)
		self.TotemBar[i]:SetBackdropColor(0, 0, 0)
		self.TotemBar[i]:SetMinMaxValues(0, 1)
						
		self.TotemBar[i].bg = self.TotemBar[i]:CreateTexture(nil, "BORDER")
		self.TotemBar[i].bg:SetAllPoints(self.TotemBar[i])
		self.TotemBar[i].bg:SetTexture(statusbar)
		self.TotemBar[i].bg.multiplier = 0.25
		
		self.TotemBar[i].Name = SetFontString(self.TotemBar[i],font,12)
		self.TotemBar[i].Name:SetPoint("LEFT",self.TotemBar[i],"LEFT",1,0)

	end
end
 	

	--== Reputation ==--
if unit == 'player' then
  if reputation == true  then
	self.Reputation = CreateFrame('StatusBar',nil,self)
	self.Reputation.MouseOver = true
	self.Reputation.Tooltip = true

	self.Reputation:SetWidth(230)
	self.Reputation:SetHeight(10)
	self.Reputation:SetPoint("TOP", self, "BOTTOM", 0, -26.75)
	self.Reputation:SetStatusBarTexture(statusbar)
	self.Reputation:SetStatusBarColor(45/255,45/255,45/255)

	self.Reputation:SetBackdrop(backdrop)
	self.Reputation:SetBackdropColor(0,0,0,0.35)


	self.Reputation.Text = SetFontString(self.Reputation, font, 12)
	self.Reputation.Text:SetPoint('TOPLEFT', self.Reputation,'TOPLEFT',8, 0)
	self.Reputation.Text:SetWidth(225)
	self.Reputation.Text:SetTextColor(1,1,1)
	self.Reputation.Text:SetJustifyH('LEFT')
  end

	--== Experience ==--
  if experience == true then 
	self.Experience = CreateFrame('StatusBar',nil,self)
	self.Experience.MouseOver = true
	self.Experience.Tooltip = true

	self.Experience:SetWidth(230)
	self.Experience:SetHeight(10)
	self.Experience:SetPoint("TOP", self, "BOTTOM", 0, -36.50)
	self.Experience:SetStatusBarTexture(statusbar)
	self.Experience:SetStatusBarColor(45/255,45/255,45/255)


	self.Experience:SetBackdrop(backdrop)
	self.Experience:SetBackdropColor(0,0,0,0.35)
	
  end
end
	
	--== oUF_PowerSpark ==--
--[[ Uncomment to enable it	
 	if unit == "player" then
 		-- Power spark!
 		local spark = pp:CreateTexture(nil, "OVERLAY")
 		spark:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark")
 		spark:SetVertexColor(1, 1, 1, 0.5)
 		spark:SetBlendMode("ADD")
 		spark:SetHeight(self.Power:GetHeight()*2)
 		spark:SetWidth(self.Power:GetHeight())
 		--  Options and settings
 		--spark.rtl = true
 		--    Make the spark go from Right To Left instead
 		--    Defaults to false
 		spark.manatick = true
 		--    Show mana regen ticks outside FSR (like the energy ticker)
 		--    Defaults to false
 		spark.highAlpha = 1
 		--    What alpha setting to use for the FSR and energy spark
 		--    Defaults to spark:GetAlpha()
 		spark.lowAlpha = 0.25
 		--    What alpha setting to use for the mana regen ticker
 		--    Defaults to highAlpha / 4
 		self.Spark = spark
 	end
]]--

--== End of plugin part ==--
	
	--== Health bar ==--
	self.Health = CreateFrame('StatusBar', nil, self)
	self.Health:SetPoint("TOPLEFT",self,"TOPLEFT",0,0)
	self.Health:SetPoint("TOPRIGHT",self,"TOPRIGHT",0,0)
	self.Health:SetStatusBarTexture(statusbar)
	self.Health:SetHeight(16)	
	self.Health:SetStatusBarColor(.3,.3,.3,1)
	
	self.Health.bg = self.Health:CreateTexture(nil, "BORDER")
	self.Health.bg:SetAllPoints(self.Health)
	self.Health.bg:SetTexture(statusbar)
	self.Health.bg:SetVertexColor(0.15,.15,0.15)
	

	--== Health bar settings ==--
	self.Health.colorTapping = false
	self.Health.colorDisconnected = true 
	self.Health.colorClass = false
	self.Health.colorClassNPC = false
	self.Health.frequentsUpdates = true

	
	--== The health text ==--
  if healthtext_opt == true then
	local healthtext = SetFontString(self.Health, font, 11) 	
	self:Tag(healthtext,'[deith:gradienthp]')
	if unit == 'target' then
		healthtext:SetWidth(130)
		healthtext:SetJustifyH('RIGHT')
		healthtext:SetPoint('RIGHT', self.Health, 'RIGHT',-2.45,0)

	else
		healthtext:SetPoint('LEFT',self.Health,'LEFT',2.45,0)
		healthtext:SetJustifyH("LEFT")
	end
  end
	
	
	--== The power bar (mana, runic power,rage and energy) ==--
	self.Power = CreateFrame('StatusBar', nil, self)
	self.Power:SetPoint("TOPLEFT", self.Health, "BOTTOMLEFT", 0,-1 )
	self.Power:SetPoint("TOPRIGHT", self.Health, "BOTTOMRIGHT", 0,-1 )
	self.Power:SetStatusBarTexture(statusbar)
	self.Power:SetHeight(5)
	self.Power:SetParent(self)
	
	self.Power.bg = self.Power:CreateTexture(nil, "BORDER")
	self.Power.bg:SetAllPoints(self.Power)
	self.Power.bg:SetTexture(statusbar)
	self.Power.bg.multiplier = 0.3

	--== The power bar settings ==--
	self.Power.colorClass = true
	self.Power.colorClassNPC = true
	self.Power.colorPower = false
	self.Power.frequentsUpdates = true


	


	--== The Power texts customisation ==--
 if powertext_opt == true then
	local powertext = SetFontString(self.Power, font, 10)

	if unit == 'player' then
		if pClass == 'DEATHKNIGHT' or pClass =='ROGUE' or pClass =='WARRIOR' then
			self:Tag(powertext,'[deith:coloredmana]') -- Will show the value not the percent
		else 
			self:Tag(powertext, '[deith:coloredmanaper]') -- Will show the percent
		end
	powertext:SetPoint('RIGHT',self.Health,'RIGHT',-2.45,0)
	powertext:SetJustifyH('RIGHT')
	end
	
  end

	--== Combo ==--
	local cp = SetFontString(self.Health, font, 14)
	self:Tag(cp,"[deith:shortcp]")
	cp:SetPoint("CENTER",self.Health,"CENTER",0,-5)
	cp:SetTextColor(unpack(oUF.colors.power["ENERGY"]))

	--== The druid mana again ==--
	if(playerClass == 'DRUID' and unit == "player") then
		self.DruidPower = CreateFrame('StatusBar', nil, self)
		self.DruidPower:SetAllPoints(self.Power)
		self.DruidPower:SetStatusBarTexture(statusbar)
		self.DruidPower:SetHeight(1)
		self.DruidPower:SetWidth(230)
		self.DruidPower:SetAlpha(0)

		self.DruidPower.Text = SetFontString(self.Power, font, 10)
		self.DruidPower.Text:SetPoint('RIGHT', self.DruidPower)
		self.DruidPower.Text:SetTextColor(unpack(colors.power['MANA']))

		table.insert(self.__elements, UpdateDruidPower)
		self:RegisterEvent('UNIT_MANA', UpdateDruidPower)
		self:RegisterEvent('UNIT_ENERGY', UpdateDruidPower)
		self:RegisterEvent('UPDATE_SHAPESHIFT_FORM', UpdateDruidPower)
	end
	

	--== Raid Icons ==--
  if RI == true then
	self.RaidIcon = self.Health:CreateTexture(nil, 'OVERLAY')
	self.RaidIcon:SetPoint('TOP', 4, 4)
	self.RaidIcon:SetHeight(16)
	self.RaidIcon:SetWidth(16)
	
   end
   
   	--== Leader Icon ==--
  if leader == true then
	self.Leader = self.Health:CreateTexture(nil, 'OVERLAY')
	self.Leader:SetPoint('RIGHT', self.RaidIcon,"LEFT", -2,0 )
	self.Leader:SetHeight(16)
	self.Leader:SetWidth(16)
  end
   

	--== The name customisation ==--
  if name_opt == true then
      if unit == "target" then
	local name = SetFontString(self.Health, font, 10) 
	name:SetJustifyH('LEFT')
	name:SetWidth(100)
	name:SetHeight(12)

	name:SetPoint('LEFT',self.Health,'LEFT',2.45,0)
	self:Tag(name,"[deith:level] [raidcolor][deith:shortname]")
		
      end
 end 
	
			

	--== Castbar ==--
		self.Castbar = CreateFrame('StatusBar', nil, self)
		self.Castbar:SetWidth(260)
		self.Castbar:SetHeight(10)
		if unit == "target" then
			self.Castbar:SetPoint("BOTTOMRIGHT", self, "TOPRIGHT",0 , 4)
		else 
			self.Castbar:SetPoint("BOTTOMLEFT", self, "TOPLEFT", 0, 4)
		end
		self.Castbar:SetStatusBarTexture(statusbar)
		self.Castbar:SetStatusBarColor(45/255,45/255,45/255)

		--== The spell casted ==--
		self.Castbar.Text = SetFontString(self.Castbar, font, 11)
		if unit == "player" then
			self.Castbar.Text:SetPoint('TOPLEFT', self.Castbar,'TOPLEFT',8, 1)
			self.Castbar.Text:SetJustifyH('LEFT')
		else
			self.Castbar.Text:SetPoint('TOPRIGHT', self.Castbar,'TOPRIGHT',-8, 1)
			self.Castbar.Text:SetJustifyH('RIGHT')
		end
		self.Castbar.Text:SetWidth(225)
		self.Castbar.Text:SetTextColor(1,1,1)
			

		--== The time to cast ==--
		self.Castbar.Time = SetFontString(self.Castbar, font, 9)
		self.Castbar.Time:SetTextColor(1,1,1)
		if unit == "player" then
			self.Castbar.Time:SetPoint('RIGHT', self.Castbar, -1, 1)
			self.Castbar.Time:SetJustifyH('RIGHT')
		else 
			self.Castbar.Time:SetPoint('LEFT', self.Castbar, 1, 1)
			self.Castbar.Time:SetJustifyH('LEFT')
		end
		--== A little background ==--
		self.Castbar:SetBackdrop(backdrop)
		self.Castbar:SetBackdropColor(0,0,0,1)

		self.Castbar.Icon = self.Castbar:CreateTexture(nil, 'ARTWORK')
		if unit == "target" then
			self.Castbar.Icon:SetPoint('TOPLEFT', self.Castbar,'BOTTOMLEFT',0,-4)
		else
			self.Castbar.Icon:SetPoint('TOPRIGHT', self.Castbar,'BOTTOMRIGHT',0,-4)
		end
		self.Castbar.Icon:SetHeight(23)
		self.Castbar.Icon:SetWidth(23)
		
		self.Castbar.Icon.Bg = self.Castbar:CreateTexture(nil,"OVERLAY")
		self.Castbar.Icon.Bg:SetTexture(iconTex)
		self.Castbar.Icon.Bg:SetVertexColor(.25,.25,.25,1)
		self.Castbar.Icon.Bg:SetPoint("TOPLEFT",self.Castbar.Icon,"TOPLEFT",-2,2)
		self.Castbar.Icon.Bg:SetPoint("BOTTOMRIGHT",self.Castbar.Icon,"BOTTOMRIGHT",2,-2)
		
		--== The safezone ==--
		if unit == "player" then
			self.Castbar.SafeZone = self.Castbar:CreateTexture(nil,'BORDER')
			self.Castbar.SafeZone:SetTexture(222/255,126/255,0,.75)
		end  
			
			--== Buffs and debuffs ==--
	if unit == "target" then 
	  if target_debuff == true then		
		self.Debuffs = CreateFrame("Frame", nil, self)
		self.Debuffs:SetHeight(23 * 0.97)
		self.Debuffs:SetWidth(230)
		self.Debuffs.size = 23 * 0.87
		self.Debuffs.spacing = 1
		self.Debuffs:SetPoint("BOTTOMLEFT", self, "TOPLEFT", -.5, 16)
		self.Debuffs.initialAnchor = "BOTTOMRIGHT"
		self.Debuffs["growth-y"] = "UP"
		self.Debuffs["growth-x"] = "LEFT"
		self.Debuffs.onlyShowPlayer = false	
		
		
		self.Buffs = CreateFrame("Frame", nil, self)
		self.Buffs:SetHeight(24)
		self.Buffs:SetWidth(24 * 8)
		self.Buffs.size = 24
		self.Buffs.spacing = 1
		self.Buffs:SetPoint("TOPLEFT", self, "TOPRIGHT", 7, -1)
		self.Buffs.initialAnchor = "TOPLEFT"
		self.Buffs["growth-y"] = "DOWN"
		
		end
	end

			--== Set the height and the width of our frames ==--

	if unit == 'player' or unit == 'target' then
		self:SetAttribute('initial-height', 22)        -- the frames' height 
	   	 self:SetAttribute('initial-width', 230)        -- the frames' width
	end
	
	
	self.PostCreateAuraIcon = createAura
	self.PostUpdateAuraIcon = updateDebuff
			
	return self
end


oUF:RegisterStyle("Deith", Style)
