--== Custom colors ==--
local colors = setmetatable({
	power = setmetatable({
		['MANA'] = {0, 144/255, 1}
	}, {__index = oUF.colors.power}),
}, {__index = oUF.colors})

local function ColorGradient(perc, ...)
	if perc >= 1 then
		local r, g, b = select(select('#', ...) - 2, ...)
		return r, g, b
	elseif perc <= 0 then
		local r, g, b = ...
		return r, g, b
	end
	
	local num = select('#', ...) / 3

	local segment, relperc = math.modf(perc*(num-1))
	local r1, g1, b1, r2, g2, b2 = select((segment*3)+1, ...)

	return r1 + (r2-r1)*relperc, g1 + (g2-g1)*relperc, b1 + (b2-b1)*relperc
end

oUF.TagEvents['deith:level'] = "UNIT_LEVEL PLAYER_LEVEL_UP UNIT_CLASSIFICATION_CHANGED"
if (not oUF.Tags['deith:level']) then
	oUF.Tags['deith:level'] = function(unit)
			local c = UnitClassification(unit)
			if(c == 'worldboss') then
				return 'Boss'
			else
				local class =oUF.Tags['shortclassification'](unit)
				local level =oUF.Tags['level'](unit)
				if(class) then
					return level .. class
				else
					return level
				end
			end
	end
end

oUF.TagEvents['deith:coloredmana']   = 'UNIT_ENERGY UNIT_FOCUS UNIT_MANA UNIT_RAGE UNIT_RUNIC_POWER'
if (not oUF.Tags['deith:coloredmana']) then
oUF.Tags['deith:coloredmana'] = function(unit)
	local r, g, b, t
	local _, ptype = UnitPowerType(unit)
	local p = UnitPower(unit)

	t = colors.power[ptype]
	if(t) then
		r, g, b = t[1], t[2], t[3]
	end
	
	return string.format("|cff%02x%02x%02x%d|r",r*255,g*255,b*255,p)

end
end


oUF.TagEvents['deith:coloredmanaper']   = "UNIT_MAXENERGY UNIT_MAXFOCUS UNIT_MAXMANA UNIT_MAXRAGE UNIT_ENERGY UNIT_FOCUS UNIT_MANA UNIT_RAGE UNIT_MAXRUNIC_POWER UNIT_RUNIC_POWER"
if (not oUF.Tags['deith:coloredmanaper']) then
oUF.Tags['deith:coloredmanaper'] = function(unit)
	local r, g, b, t
	local _, ptype = UnitPowerType(unit)
	local p = UnitPower(unit)
	local pmax = UnitPowerMax(unit)

	t = colors.power[ptype]
	if(t) then
		r, g, b = t[1], t[2], t[3]
	end
	
	return string.format("|cff%02x%02x%02x%d/%d|r",r*255,g*255,b*255,p,pmax)

end
end

local numberize = function(v)
	if v <= 9999 then return v end
	if v >= 1000000 then
		local value = string.format("%.1fm", v/1000000)
		return value
	elseif v >= 10000 then
		local value = string.format("%.1fk", v/1000)
		return value
	end
end

oUF.TagEvents['deith:gradienthp']   = "UNIT_HEALTH UNIT_MAXHEALTH"
if (not oUF.Tags['deith:gradienthp']) then
oUF.Tags['deith:gradienthp'] = function(unit)
	local cur = UnitHealth(unit)
	local max = UnitHealthMax(unit)
	--local cfm, mfm


	local r, g, b	
	r, g, b = ColorGradient(cur/max, 0.69, 0.31, 0.31, 0.65, 0.63, 0.35, 0.33, 0.59, 0.33)
	
	return string.format("|cff%02x%02x%02x"..numberize(cur).."/"..numberize(max).."|r",r*255,g*255,b*255)

end
end


oUF.TagEvents['deith:shortname'] = 'UNIT_NAME_UPDATE'
if (not oUF.Tags['deith:shortname']) then
oUF.Tags['deith:shortname'] = function(unit)	
	local name = UnitName(unit)
	return (string.len(name) > 10) and string.gsub(name, "%s?(.)%S+%s", "%1. ") or name
end
end


oUF.TagEvents['deith:shortcp'] = 'UNIT_COMBO_POINTS UNIT_TARGET'
if (not oUF.Tags['deith:shortcp']) then
oUF.Tags['deith:shortcp'] = function(unit)	
	local cp = GetComboPoints(unit, 'target') return (cp > 0) and strrep("* ",cp) end
	--return strrep("* ", 5) end -- test purpose
end
